@@ -38,6 +38,7 @@ void test_filter_null(void);
 void test_bigram_split_symbol_tokenizer(void);
 void test_nonexistent_table(void);
 void test_boolean(void);
+void test_equal_index(void);
 
 static gchar *tmp_directory;
 
@@ -477,3 +478,27 @@ test_boolean(void)
                                        "--output_columns _key,public "
                                        "--filter public==true"));
 }
+
+void
+test_equal_index(void)
+{
+  assert_send_command("table_create Blogs TABLE_HASH_KEY ShortText");
+  assert_send_command("table_create Titles TABLE_HASH_KEY ShortText");
+  assert_send_command("column_create Blogs title COLUMN_SCALAR Titles");
+  assert_send_command("column_create Titles Blogs_title "
+                      "COLUMN_INDEX Blogs title");
+
+  assert_send_command("load --table Blogs --columns '_key, title' \n"
+                      "[\n"
+                      " [\"groonga-1-0\", \"groonga 1.0 release!\"],\n"
+                      " [\"groonga-2-0\", \"groonga 2.0 release!\"]\n"
+                      "]");
+  cut_assert_equal_string(
+    "[[[1],"
+     "[[\"_key\",\"ShortText\"],"
+      "[\"title\",\"Titles\"]],"
+      "[\"groonga-1-0\",\"groonga 1.0 release!\"]]]",
+    send_command("select Blogs "
+                 "--output_columns _key,title "
+                 "--filter 'title == \"groonga 1.0 release!\"'"));
+}
