@@ -51,8 +51,7 @@ void test_valid(gconstpointer data);
 void
 cut_startup(void)
 {
-  tmp_directory = g_build_filename(grn_test_get_base_dir(),
-                                   "tmp",
+  tmp_directory = g_build_filename(grn_test_get_tmp_dir(),
                                    "test-expr-script",
                                    NULL);
 }
