@@ -33,8 +33,7 @@ static grn_obj *database;
 void
 cut_startup(void)
 {
-  tmp_directory = g_build_filename(grn_test_get_base_dir(),
-                                   "tmp",
+  tmp_directory = g_build_filename(grn_test_get_tmp_dir(),
                                    "register",
                                    NULL);
 }
@@ -117,9 +116,9 @@ test_register_function(void)
                       "COLUMN_INDEX|WITH_POSITION Sites _key");
   assert_send_command("load '[[\"_key\"],[\"groonga.org\"]]' Sites");
   cut_assert_equal_string("[[[1],[[\"_score\",\"Int32\"]],[11]]]",
-                          send_command("select Sites "                  \
-                                       "--output_columns _score "       \
-                                       "--match_columns _key "          \
-                                       "--query groonga "               \
+                          send_command("select Sites "
+                                       "--output_columns _score "
+                                       "--match_columns _key "
+                                       "--query groonga "
                                        "--scorer '_score=str_len(_key)'"));
 }
