@@ -218,6 +218,17 @@ grn_test_get_build_dir(void)
   return build_dir;
 }
 
+static gchar *tmp_dir = NULL;
+const gchar *
+grn_test_get_tmp_dir(void)
+{
+  if (!tmp_dir) {
+    tmp_dir = g_build_filename(grn_test_get_build_dir(), "tmp", NULL);
+  }
+
+  return tmp_dir;
+}
+
 typedef struct _grn_log
 {
   gint level;
@@ -263,6 +274,7 @@ typedef struct _grn_logger_context
   grn_logger_info *logger;
   GList *logs;
   GList *messages;
+  GString *collected_message;
 } grn_logger_context;
 
 static grn_logger_context *
@@ -274,6 +286,7 @@ grn_logger_context_new(grn_logger_info *logger)
   context->logger = logger;
   context->logs = NULL;
   context->messages = NULL;
+  context->collected_message = NULL;
 
   return context;
 }
@@ -284,6 +297,10 @@ grn_logger_context_clear_messages(grn_logger_context *context)
   g_list_foreach(context->messages, (GFunc)g_free, NULL);
   g_list_free(context->messages);
   context->messages = NULL;
+  if (context->collected_message) {
+    g_string_free(context->collected_message, TRUE);
+    context->collected_message = NULL;
+  }
 }
 
 static void
@@ -310,8 +327,12 @@ grn_collect_logger_log_func(int level, const char *time, const char *title,
   grn_log *log;
 
   log = grn_log_new(level, time, title, message, location);
-  context->logs = g_list_prepend(context->logs, log);
-  context->messages = g_list_prepend(context->messages, g_strdup(message));
+  context->logs = g_list_append(context->logs, log);
+  context->messages = g_list_append(context->messages, g_strdup(message));
+  if (context->collected_message) {
+    g_string_free(context->collected_message, TRUE);
+    context->collected_message = NULL;
+  }
 }
 
 grn_logger_info *
@@ -344,21 +365,25 @@ grn_collect_logger_get_messages(grn_logger_info *logger)
   return context->messages;
 }
 
-gchar *
+const gchar *
 grn_collect_logger_to_string(grn_logger_info *logger)
 {
-  GString *string;
+  grn_logger_context *context = logger->func_arg;
   const GList *messages;
 
-  string = g_string_new(NULL);
+  if (context->collected_message) {
+    return context->collected_message->str;
+  }
+
+  context->collected_message = g_string_new(NULL);
   for (messages = grn_collect_logger_get_messages(logger);
        messages;
        messages = g_list_next(messages)) {
     const gchar *message = messages->data;
-    g_string_append_printf(string, "%s\n", message);
+    g_string_append_printf(context->collected_message, "%s\n", message);
   }
 
-  return g_string_free(string, FALSE);
+  return context->collected_message->str;
 }
 
 void
@@ -606,12 +631,7 @@ const gchar *
 grn_test_send_command(grn_ctx *context, const gchar *command)
 {
   unsigned int send_id, receive_id;
-  int flags = 0;
-  grn_rc rc = GRN_SUCCESS;
-  gchar *result_status_end;
   GString *result;
-  const gchar *result_status_start_mark = "[[";
-  const gchar *result_status_end_mark = "],";
   const gchar **lines;
 
   result = g_string_new(NULL);
@@ -619,37 +639,17 @@ grn_test_send_command(grn_ctx *context, const gchar *command)
   for (; *lines; lines++) {
     gchar *command_result;
     unsigned int command_result_length;
+    int flags = 0;
 
     send_id = grn_ctx_send(context, *lines, strlen(*lines), 0);
     receive_id = grn_ctx_recv(context, &command_result, &command_result_length,
                               &flags);
     cut_assert_equal_uint(send_id, receive_id);
     g_string_append_len(result, command_result, command_result_length);
+    grn_test_assert_context(context,
+                            cut_message("<%s>:<%s>", command, result->str));
   }
 
-  cut_assert_not_equal_uint(0, result->len, cut_message("<%s>", command));
-  if (g_str_has_prefix(result->str, result_status_start_mark)) {
-    const gchar *result_status_start;
-    const gchar *rest;
-
-    result_status_start = result->str + strlen(result_status_start_mark);
-    rc = grn_atoi(result_status_start, result->str + result->len, &rest);
-    cut_assert_not_equal_string(result_status_start, rest);
-    grn_test_assert(rc, cut_message("<%s>", result->str));
-  }
-
-  result_status_end = g_strstr_len(result->str, result->len,
-                                   result_status_end_mark);
-  if (result_status_end) {
-    const gchar *result_end_mark = "]";
-
-    g_string_erase(result,
-                   0,
-                   result_status_end - result->str +
-                   strlen(result_status_end_mark));
-    g_string_truncate(result,
-                      result->len - strlen(result_end_mark));
-  }
   return cut_take_strdup(g_string_free(result, FALSE));
 }
 
